from .__about__ import __application__, __author__, __version__
from ._tool import LangchainRetrievalTool

__all__ = [
    "__version__",
    "__application__",
    "__author__",
    "LangchainRetrievalTool",
]



import wikipedia
from google.adk.agents import LlmAgent
from langchain_community.retrievers import WikipediaRetriever

from adk_retrieval import LangchainRetrievalTool

root_agent = LlmAgent(
    name="wiki_basic",
    description="Agent for summarizing Wikipedia articles.",
    instruction="""You are a specialized search & summarizer agent for Wikipedia articles.
      """,
    model="gemini-2.5-flash",
    tools=[
        LangchainRetrievalTool(
            name="wikipedia_retrieval_tool",
            description="Tool for retrieving Wikipedia articles based on user queries.",
            retriever=WikipediaRetriever(wiki_client=wikipedia),
        )
    ],
)



import wikipedia
from google.adk.agents import LlmAgent
from langchain_community.retrievers import WikipediaRetriever
from langchain_core.documents import Document

from adk_retrieval import LangchainRetrievalTool


# an example post-processing function that combines multiple documents into a single string
async def post_process(docs: list[Document]) -> str:
    sources = []
    for index, doc in enumerate(docs):
        sources.append(f"Source {index + 1}:\n{doc.page_content}\n")

    return "\n\n".join(sources)


root_agent = LlmAgent(
    name="wiki_sophisticated",
    description="Agent for summarizing Wikipedia articles.",
    instruction="""You are a specialized search & summarizer agent for Wikipedia articles.
      """,
    model="gemini-2.5-flash",
    tools=[
        LangchainRetrievalTool(
            name="wikipedia_retrieval_tool",
            description="Tool for retrieving Wikipedia articles based on user queries.",
            retriever=WikipediaRetriever(wiki_client=wikipedia, top_k_results=3),
            post_process=post_process,
        )
    ],
)



from typing import Any, Awaitable, Callable

from google.adk.tools.retrieval import BaseRetrievalTool
from google.adk.tools.tool_context import ToolContext
from langchain_core.documents import Document
from langchain_core.retrievers import BaseRetriever
from typing_extensions import override


class LangchainRetrievalTool(BaseRetrievalTool):
    """A retrieval tool that uses Langchain for document retrieval."""

    def __init__(
        self,
        name: str,
        description: str,
        retriever: BaseRetriever,
        post_process: Callable[[list[Document]], Awaitable[str]] | None = None,
    ):
        super().__init__(name=name, description=description)
        self._retriever = retriever
        self._post_process = post_process

    @override
    async def run_async(self, *, args: dict[str, Any], tool_context: ToolContext) -> Any:
        query = args["query"]
        documents = await self._retriever.ainvoke(query)
        if self._post_process:
            return await self._post_process(documents)
        return documents[0].page_content
